// File: SocketThrdServer.java
// TCP/IP threaded Java server example

import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.event.*;
import javax.swing.*;

import java.io.*;
import java.net.*;

class ClientWorker implements Runnable {
  private Socket client;
  private JTextArea textArea;
  
  ClientWorker(Socket client, JTextArea textArea) {
   this.client = client;
   this.textArea = textArea;   
  }

  public void run(){
    String line;
    BufferedReader in = null;
    PrintWriter out = null;
    // Set up streams
    try{
      in = new BufferedReader(new InputStreamReader(client.getInputStream()));
      out = new PrintWriter(client.getOutputStream(), true);
    } 
    catch (IOException e) {
      System.out.println("in or out failed");
      System.exit(-1);
    }
    // Communications
    while(true){
      	try{
         line = in.readLine();
	 //Send data back to client
         out.println(line);
         textArea.append(line+'\n');
      	} 
	catch (IOException e) {
         System.out.println("Read failed");
         System.exit(-1);
       	}
    }
  }
}

class SocketThrdServer extends JFrame{

   JLabel label = new JLabel("Text received over socket:");
   JPanel panel;
   JTextArea textArea;
   ServerSocket server = null;
   int port;

   SocketThrdServer(){ //Begin Constructor
     textArea = new JTextArea();
     JScrollPane textScroll=new JScrollPane(textArea);
     textScroll.setSize(200,100);
     panel = new JPanel();
     panel.setLayout(new BorderLayout());
     panel.setBackground(Color.white);
     getContentPane().add(panel);
     panel.add("North", label);
     panel.add("Center", textScroll);
     setBounds(200,200,200,140);
     setVisible(true);
   } //End Constructor

   public void listenSocket(int port)
   {// First, get a socket; use the port passed in
    try{
      server = new ServerSocket(port); 
    } 
    catch (IOException e) {
      System.out.println("Could not listen on port "+port);
      System.exit(-1);
    }
    // Handle service requests
    while(true){
      ClientWorker w;
      try{
	// accept() fires when a request comes in; blocks until then
        w = new ClientWorker(server.accept(), textArea);
	// Set up a thread to handle this client's requests
        Thread t = new Thread(w);
	// execute the thread's run() method;
        t.start();    
	// then, back to wait for another client's request
      } 
      catch (IOException e) {
        System.out.println("Accept failed: Port #"+port);
        System.exit(-1);
      }
    }
  }

  protected void finalize(){
  //Objects created in run method are finalized when 
  //program terminates and thread exits
     try{
        server.close();
    } 
    catch (IOException e) {
        System.out.println("Could not close socket");
        System.exit(-1);
    }
  }

  public static void main(String[] args){
        SocketThrdServer frame = new SocketThrdServer();
	frame.setTitle("Server Program");
        WindowListener l = new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                        System.exit(0);
                }
        };
        frame.addWindowListener(l);
//        frame.pack();
        frame.setVisible(true);
        int port=0;
	if (args.length==0) {
          System.out.print("Enter a port number for the server (bigger than 1250) >");
     	  BufferedReader in=new BufferedReader(new InputStreamReader(System.in));
     	  try {
       		port=Integer.parseInt(in.readLine());
     	  }
     	  catch (Exception noPort) {
       	    	System.out.println("Port not read");
       		System.exit(0);
     	  }
	}
	else try {
	  port=Integer.parseInt(args[0]);
	}
   	catch (Exception noPort) {
       	  System.out.println("Invalid Command Line Argument");
       	  System.exit(0);
     	}
	// Frame is ready; let it begin being a server
        frame.listenSocket(port);
  }
}

